#!/bin/bash
#
#  Copyright FUJITSU LIMITED 2013
#
#  UNINSTALL.sh - uninstall script
#
export LANG=C
PATH=/bin:/usr/bin:/usr/sbin:$PATH

# get shell path/name
CURDIR=`pwd`
cd `dirname $0`
SH_PATH=`pwd`
cd $CURDIR
SH_NAME=`basename $0`

# Install Definition
RPMSPATH="RPMS"
SOFTNAME=`basename $SH_PATH`
PKGS=`find $SH_PATH/$RPMSPATH -name "*.rpm" -print 2>/dev/null`
PKGS=`rpm -qp --qf '%{NAME}\n' $PKGS `

# Environment Definition
STDOUT="/dev/stdout"

# LOG file
MSGLOG="/tmp/FJSVdr-util-setup.log"
MSGID="FJSVdr-util"

#  Install time
DATE=`date '+%b %d %Y'`

# Initialize
RESULT=0
F_FORCE=0
S_INSTALL=0
ARCH=`uname -i`

#
# Output message function
#   usage: msg msg-text
#
msg()
{
	echo "$MSGID : $*" | tee -a $MSGLOG
}

#
# Shell exit function
#   usage: shell_exit [code]
#
shell_exit()
{
	ACTION="uninstall"
	if [ $1 != 0 ]; then
		msg "$SOFTNAME $ACTION failed."
		ret=1
	else
		msg "$SOFTNAME $ACTION completed."
		ret=0
	fi
        echo "" >> $MSGLOG
	exit $ret
}

do_rpm()
{
	msg "#" rpm "$@"
	rpm "$@" 2>&1 | tee -a $MSGLOG
	ret=${PIPESTATUS[0]}
	return $ret
}

#
# uninstall function
#   usage: uninstall rpm-file ...
#   result code: =0(completed), !=0(failed)
#
uninstall()
{
	# set rpm options
	OPT=""
	if [ $F_FORCE != 0 ]; then
		OPT="$OPT --nodeps"
	fi
	# execute rpm
	do_rpm -e $OPT $*
	return $?
}

#
# usage
#   usage: usage
#
usage()
{
	msg "Usage: $SH_NAME [--force]"
}


#
# Main routine
#

# user account check (root)
if [ $EUID != 0 ]; then
	msg "$SH_NAME: only root can do that."
	shell_exit 1
fi

# check argument
for ARG in $*; do
	case $ARG in
	"--force")
		F_FORCE=1
		;;
	*)
		msg "$SH_NAME: option $ARG invalid."
		usage
		shell_exit 1
		;;
	esac
done

echo "$MSGID : FJSVdr-util uninstall start at $DATE" >> $MSGLOG

# package loop
for PKG in $PKGS; do

	# check/get installed package
	S_INSTALL=0
	NOWPKG=`rpm -q $PKG 2>&1` && S_INSTALL=1

	# check obsoletes package is installed
	O_INSTALL=0
	PKGNAME=`find $SH_PATH/$RPMSPATH -name "$PKG*.rpm" -print 2>/dev/null`
	OBSPKG=`rpm -qp --obsoletes $PKGNAME 2>/dev/null`
	OBSPKG=`rpm -q $OBSPKG 2>/dev/null` && O_INSTALL=1

	# package uninstall (if package is installed)
	if [ $S_INSTALL != 0 ]; then
		uninstall $NOWPKG
		if [ $? = 0 ]; then
			msg "package $NOWPKG is uninstalled."
		else
			msg "package $NOWPKG is uninstall failed."
			RESULT=1
		fi
	elif [ $O_INSTALL != 0 ]; then
		uninstall $OBSPKG
		if [ $? = 0 ]; then
			msg "package $OBSPKG is uninstalled."
		else
			msg "package $OBSPKG is uninstall failed."
			RESULT=1
		fi
	# package not uninstall (if package is not installed)
	else
		msg "package $PKG is not installed."
	fi

done

# exit (and output message)
shell_exit $RESULT

