#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# library for version compatibility
#
# Copyright(C) 2013 MUNEDA Takahiro
# Copyright(C) 2013 FUJITSU LIMITED
#

import sys

import dp


def main_parse_args():
    # dummy
    return (None, None, False)


def add_parse_args():
    # dummy
    return None


def report_parse_args():
    # dummy
    return None


def rm_parse_args():
    # dummy
    return None


def show_parse_args():
    # dummy
    return (None, False)


def stat_parse_args():
    # dummy
    return None


try:
    import argparse
except ImportError as e:
    # this is running on RHEL6 system with python 2.6.x
    # use optparse instead
    import optparse

    def main_optparse():
        '''
        parse arguments provided for sbin/dp
        Return value(tuple (str, str, bool)): (subcommand, device, possible)
        if those are not set, None or False will be returned
        '''
        subcommands = ['add', 'rm', 'stat', 'show', 'report']

        usage = (
            "%prog [-h] [-v] {add,rm,stat,show,report} ...\n"
            '\n'
            'DR command user interface for PRIMEQUEST\n'
            '\n'
            'positional arguments:\n'
            '  {add,rm,stat,show,report}\n'
            '    add                 Device hot-add\n'
            '    rm                  Device hot-remove\n'
            '    stat                Show device status\n'
            '    show                Show resource amount\n'
            '    report              Gather system information related to DR')

        parser = optparse.OptionParser(usage=usage)
        parser.add_option(
            '-v', '--version',
            action='store_true', default=False, dest='version',
            help='show program\'s version number and exit')
        major, minor = dp.CheckDistroVersion()
        if major == '6' and not int(minor) < 5:
            parser.add_option(
                '-p', '--possible',
                action='store_true', default=False, dest='possible',
                help=('Show all resources including currently disabled.  '
                      'If subcommand is not show or device is not SBx, '
                      'will be ignored.'))

        subcommand = None
        device = None
        possible = False
        try:
            options, args = parser.parse_args()
        except SystemExit as e:
            dp.CleanupPIDFile()
            sys.exit(1)

        if options.version:
            sys.stdout.write("%s\n" % dp.PrintVersion())
            dp.CleanupPIDFile()
            sys.exit(0)
        if major == '6' and not int(minor) < 5:
            possible = options.possible

        if not 0 < len(args) < 3:
            dp.LogErr("240 : %s: Invalid argument" % (' ').join(sys.argv))
            dp.CleanupPIDFile()
            sys.exit(1)
        if args[0] not in subcommands:
            dp.LogErr("260 : %s: invalid subcommand is set" % args[0])
            dp.CleanupPIDFile()
            sys.exit(1)
        subcommand = args[0]

        if len(args) == 1 and not subcommand == 'report':
            dp.LogErr('270 : None: device is not set')
            dp.CleanupPIDFile()
            sys.exit(1)
        elif len(args) == 2 and subcommand == 'report':
            dp.LogErr("249 : %s: Invalid argument" % (' ').join(sys.argv))
            dp.CleanupPIDFile()
            sys.exit(1)
        elif not subcommand == 'report':
            device = args[1]

        return (subcommand, device, possible)

    def add_optparse():
        '''
        parse arguments provided for libexec/dp_core/dp_add.py
        Return value(str): device or None
        '''
        usage = (
            "%prog [-h] [-v] device\n"
            '\n'
            'Device hot-add\n'
            '\n'
            'positional arguments:\n'
            '  device         Target Device: {IOUx or PCIex}\n')

        parser = optparse.OptionParser(usage=usage)
        parser.add_option(
            '-v', '--version',
            action='store_true', default=False, dest='version',
            help='show program\'s version number and exit')
        try:
            options, args = parser.parse_args()
        except SystemExit as e:
            dp.CleanupPIDFile()
            sys.exit(1)

        if options.version:
            sys.stdout.write("%s\n" % dp.PrintVersion())
            dp.CleanupPIDFile()
            sys.exit(0)

        if not len(args) == 1:
            dp.LogErr("241 : %s: Invalid argument" % (' ').join(sys.argv))
            dp.CleanupPIDFile()
            sys.exit(1)
        return args[0]

    def report_optparse():
        '''
        parse arguments provided for libexec/dp_core/dp_report.py
        Return value: None
        '''
        usage = (
            "%prog [-h] [-v]\n"
            '\n'
            'Gather system information related to DR\n'
            '\n')

        parser = optparse.OptionParser(usage=usage)
        parser.add_option(
            '-v', '--version',
            action='store_true', default=False, dest='version',
            help='show program\'s version number and exit')
        try:
            options, args = parser.parse_args()
        except SystemExit as e:
            dp.CleanupPIDFile()
            sys.exit(1)

        if options.version:
            sys.stdout.write("%s\n" % dp.PrintVersion())
            dp.CleanupPIDFile()
            sys.exit(0)

        if not len(args) == 0:
            dp.LogErr("242 : %s: Invalid argument" % (' ').join(sys.argv))
            dp.CleanupPIDFile()
            sys.exit(1)
        return None

    def rm_optparse():
        '''
        parse arguments provided for libexec/dp_core/dp_rm.py
        Return value(str): device or None
        '''
        usage = (
            "%prog [-h] [-v] device\n"
            '\n'
            'Device hot-remove\n'
            '\n'
            'positional arguments:\n'
            '  device         Target Device: {IOUx or PCIex}\n')

        parser = optparse.OptionParser(usage=usage)
        parser.add_option(
            '-v', '--version',
            action='store_true', default=False, dest='version',
            help='show program\'s version number and exit')
        try:
            options, args = parser.parse_args()
        except SystemExit as e:
            dp.CleanupPIDFile()
            sys.exit(1)

        if options.version:
            sys.stdout.write("%s\n" % dp.PrintVersion())
            dp.CleanupPIDFile()
            sys.exit(0)

        if not len(args) == 1:
            dp.LogErr("243 : %s: Invalid argument" % (' ').join(sys.argv))
            dp.CleanupPIDFile()
            sys.exit(1)
        return args[0]

    def show_optparse():
        '''
        parse arguments provided for libexec/dp_core/dp_show.py
        Return value(tuple (str, bool)): (device, possible)
        '''
        usage = (
            "%prog [-h] [-v] [-p] device\n"
            '\n'
            'Show resource amount\n'
            '\n'
            'positional arguments:\n'
            '  device          Target Device: {SBx, IOUx, PCIex, PCIe}\n')

        parser = optparse.OptionParser(usage=usage)
        parser.add_option(
            '-v', '--version',
            action='store_true', default=False, dest='version',
            help='show program\'s version number and exit')
        major, minor = dp.CheckDistroVersion()
        if major == '6' and not int(minor) < 5:
            parser.add_option(
                '-p', '--possible',
                action='store_true', default=False, dest='possible',
                help=('[SB Only]: Show all resources including currently '
                      'disabled.  If subcommand is not show or device is not '
                      'SBx, will be ignored.'))

        try:
            options, args = parser.parse_args()
        except SystemExit as e:
            dp.CleanupPIDFile()
            sys.exit(1)

        if options.version:
            sys.stdout.write("%s\n" % dp.PrintVersion())
            dp.CleanupPIDFile()
            sys.exit(0)

        if not len(args) == 1:
            dp.LogErr("244 : %s: Invalid argument" % (' ').join(sys.argv))
            dp.CleanupPIDFile()
            sys.exit(1)
        if major == '6' and not int(minor) < 5:
            return (args[0], options.possible)
        else:
            return (args[0], False)

    def stat_optparse():
        '''
        parse arguments provided for libexec/dp_core/dp_stat.py
        Return value(str): device
        '''
        usage = (
            "%prog [-h] [-v] device\n"
            '\n'
            'Show device status\n'
            '\n'
            'positional arguments:\n'
            '  device         Target Device: {SB, CPU, MEM, IOU, PCIe}\n')

        parser = optparse.OptionParser(usage=usage)
        parser.add_option(
            '-v', '--version',
            action='store_true', default=False, dest='version',
            help='show program\'s version number and exit')

        try:
            options, args = parser.parse_args()
        except SystemExit as e:
            dp.CleanupPIDFile()
            sys.exit(1)

        if options.version:
            sys.stdout.write("%s\n" % dp.PrintVersion())
            dp.CleanupPIDFile()
            sys.exit(0)

        if not len(args) == 1:
            dp.LogErr("245 : %s: Invalid argument" % (' ').join(sys.argv))
            dp.CleanupPIDFile()
            sys.exit(1)
        return args[0]

    # set methods by overriding
    main_parse_args = main_optparse
    add_parse_args = add_optparse
    report_parse_args = report_optparse
    rm_parse_args = rm_optparse
    show_parse_args = show_optparse
    stat_parse_args = stat_optparse

else:
    # this is running on RHEL7 system with python 2.7.x
    # use argparse
    def main_argparse():
        '''
        parse arguments provided for sbin/dp
        Return value(tuple (str, str, bool)): (subcommand, device, possible)
        if those are not set, None or False is set
        '''
        argument_description = 'DR command user interface for PRIMEQUEST'
        parser = argparse.ArgumentParser(description=argument_description)
        parser.add_argument('-v', '--version', action='version',
                            version="%s" % dp.PrintVersion())
        subparsers = parser.add_subparsers(dest='subcommand')

        parser_add = subparsers.add_parser('add', help='Device hot-add')
        parser_add.add_argument(
            'device', help='Target Device: {IOUx or PCIex}')
        parser_rm = subparsers.add_parser('rm', help='Device hot-remove')
        parser_rm.add_argument(
            'device', help='Target Device: {IOUx or PCIex}')
        parser_stat = subparsers.add_parser('stat', help='Show device status')
        parser_stat.add_argument(
            'device', help='Target Device type: {SB, CPU, MEM, IOU, PCIe}')
        parser_show = subparsers.add_parser('show',
                                            help='Show resource amount')
        parser_show.add_argument(
            'device', help='Target Device: {SBx, IOUx, PCIex, PCIe}')
        parser_show.add_argument(
            '-p', '--possible', action='store_true',
            help=('SB Only: Show all resources including currently disabled. '
                  'If device is not SBx, will be ignored.'))
        subparsers.add_parser(
            'report', help='Gather system information related to DR')

        subcommand = None
        device = None
        possible = False
        try:
            args = parser.parse_args()
        except SystemExit:
            dp.CleanupPIDFile()
            sys.exit(1)

        subcommand = args.subcommand
        if not args.subcommand == 'report':
            device = args.device
        if args.subcommand == 'show':
            possible = args.possible

        return (subcommand, device, possible)

    def add_argparse():
        '''
        parse arguments provided for libexec/dp_core/dp_add.py
        Return value(str): device
        '''
        argument_description = 'Device hot-add'
        parser = argparse.ArgumentParser(description=argument_description)
        parser.add_argument('-v', '--version', action='version',
                            version="%s" % dp.PrintVersion())
        parser.add_argument('device', help='Target Device: {IOUx or PCIex}')

        try:
            args = parser.parse_args()
        except SystemExit:
            dp.CleanupPIDFile()
            sys.exit(1)
        return args.device

    def report_argparse():
        '''
        parse arguments provided for libexec/dp_core/dp_report.py
        Return value: None
        '''
        argument_description = 'Gather system information related to DR'
        parser = argparse.ArgumentParser(description=argument_description)
        parser.add_argument('-v', '--version', action='version',
                            version="%s" % dp.PrintVersion())

        try:
            parser.parse_args()
        except SystemExit:
            dp.CleanupPIDFile()
            sys.exit(1)
        return None

    def rm_argparse():
        '''
        parse arguments provided for libexec/dp_core/dp_rm.py
        Return value(str): device
        '''
        argument_description = 'Device hot-remove'
        parser = argparse.ArgumentParser(description=argument_description)
        parser.add_argument('-v', '--version', action='version',
                            version="%s" % dp.PrintVersion())
        parser.add_argument('device', help='Target Device: {IOUx or PCIex}')

        try:
            args = parser.parse_args()
        except SystemExit:
            dp.CleanupPIDFile()
            sys.exit(1)
        return args.device

    def show_argparse():
        '''
        parse arguments provided for libexec/dp_core/dp_show.py
        Return value(tuple (str, bool)): (device, possible)
        '''
        argument_description = 'Show resource amount'
        parser = argparse.ArgumentParser(description=argument_description)
        parser.add_argument('-v', '--version', action='version',
                            version="%s" % dp.PrintVersion())
        parser.add_argument('device',
                            help='Target Device: {SBx, IOUx, PCIex, PCIe}')
        parser.add_argument('-p', '--possible', action='store_true',
                            help=('[SB Only]: Show all resources including '
                                  'currently disabled. If device is not SBx, '
                                  'this option will be ignored.'))

        try:
            args = parser.parse_args()
        except SystemExit:
            dp.CleanupPIDFile()
            sys.exit(1)
        return (args.device, args.possible)

    def stat_argparse():
        '''
        parse arguments provided for libexec/dp_core/dp_stat.py
        Return value(str): device
        '''
        argument_description = 'Show device status'
        parser = argparse.ArgumentParser(description=argument_description)
        parser.add_argument('-v', '--version', action='version',
                            version="%s" % dp.PrintVersion())
        parser.add_argument('device',
                            help='Target Device: {SB, CPU, MEM, IOU, PCIe}')

        try:
            args = parser.parse_args()
        except SystemExit:
            dp.CleanupPIDFile()
            sys.exit(1)
        return args.device

    # set methods by overriding
    main_parse_args = main_argparse
    add_parse_args = add_argparse
    report_parse_args = report_argparse
    rm_parse_args = rm_argparse
    show_parse_args = show_argparse
    stat_parse_args = stat_argparse
