#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# library for DR event notification
#
# Copyright(C) 2013 MUNEDA Takahiro
# Copyright(C) 2013 FUJITSU LIMITED
#

import os
import errno
import socket

import dp


DP_SOCKET_PATH = '/var/run/dp.socket'
DP_SOCKET_TIMEOUT = 10  # seconds


def _OpenDPSocket(server):
    '''
    Prepare a UNIX domain socket to commucate for DR event notify.
    The returned socket is ready to communicate.
    @server(int): 0 -> client, other than 0 -> server
    Return value(socket_object): socket object.  If failed, 'None'
    '''
    if not isinstance(server, int):
        dp.LogDebug("%s: wrong server was passed" % str(server))
        return None

    if server:
        # Remove an outdated socket if exists
        # It may exist if the last session was terminated abnormally
        try:
            os.unlink(DP_SOCKET_PATH)
        except OSError as e:
            if not e.errno == errno.ENOENT:
                dp.LogDebug(e)
                return None

    try:
        s = socket.socket(socket.AF_UNIX)
        if server:
            s.bind(DP_SOCKET_PATH)
            s.listen(1)
        else:
            s.connect(DP_SOCKET_PATH)
    except Exception as e:
        dp.LogDebug(e)
        return None
    return s


def _CloseDPSocket(s, server):
    '''
    Close a UNIX domain socket to communicate for DR event notify
    @s(socket_object): UNIX domain socket to communicate
    @server(int): 0 -> client, other than 0 -> server
    Return value(int): success -> 0, error -> 1
    '''
    if not s or not isinstance(s, socket._socketobject):
        dp.LogDebug("%s: wrong socket was passed" % str(s))
        return 1
    if not isinstance(server, int):
        dp.LogDebug("%s: wrong server was passed" % str(server))
        return 1

    try:
        s.close()
        if server:
            os.unlink(DP_SOCKET_PATH)
    except Exception as e:
        dp.LogDebug(e)
        return 1
    return 0
