#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# dp add function
#
# Copyright(C) 2012 MUNEDA Takahiro
# Copyright(C) 2012 FUJITSU LIMITED
#

import os
import re
import sys

import dp
import ipmi
import dp_compat


def main(device):
    '''
    Call Add() for corresponding device
    @device(str): device to be added
    Return value(int): success -> 0, error -> 1
    '''
    if not device or not isinstance(device, str):
        if isinstance(device, tuple):
            device = (device,)
        dp.LogDebug("%s: wrong device was passed" % device)
        dp.LogErr("210 : %s: invalid device is set" % device)
        return 1

    dev = device.lower()
    if re.match(r'sb\d+$', dev):
        dev_type = 'sb'
        dev_num = int(re.sub(r'^sb', '', dev))
        if not 0 <= dev_num < dp.DP_SB_MAX:
            dp.LogErr("211 : %s: invalid device is set" % device)
            return 1
    elif re.search(r'ACPI0004:', device):
        # for RHEL6.4 compatibility
        dev_type = 'sb'
        dev_num = -1
    elif re.match(r'iou\d+$', dev):
        dev_type = 'iou'
        dev_num = int(re.sub(r'^iou', '', dev))
        if not 0 <= dev_num < dp.DP_IOU_MAX:
            dp.LogErr("212 : %s: invalid device is set" % device)
            return 1
    elif re.match(r'pcie\d+$', dev):
        dev_type = 'pcie'
        dev_num = int(re.sub(r'^pcie', '', dev))
        if not os.path.isdir("%s/bus/pci/slots/%s" % (dp.DP_SYSFS, dev_num)):
            dp.LogErr("213 : %s: invalid device is set" % device)
            return 1
    else:
        dp.LogErr("214 : %s: invalid device is set" % device)
        return 1

    if dev_type == 'sb':
        sb = dp.SB(dev_num)
        if dev_num == -1:
            sb.devpath = device
        retval = sb.Add()
        if ipmi.SendSBAddCompletion(retval):
            dp.LogErr('600 : Failed to send IPMI Command: Completion')
        if retval:
            return 1
        else:
            return 0
    elif dev_type == 'iou':
        iou = dp.IOU(dev_num)
        retval = iou.Add()
        pci_devs = dp.ScanPciDevsOnSystem()
        if ipmi.SendBusTreeStart('IOU_ADD') == 0:
            ipmi.SendBusTrees(pci_devs)
            ipmi.SendBusTreeEnd('IOU_ADD')
        ipmi.SendMacAddresses(pci_devs)
        if ipmi.SendIOUAddCompletion(retval):
            dp.LogErr('601 : Failed to send IPMI Command: Completion')
        if retval:
            return 1
        else:
            return 0
    elif dev_type == 'pcie':
        pcie = dp.PciHotPlugSlot(dev_num)
        return pcie.Add()


def pre_main():
    '''
    Check command argument and system environment before processing main().
    Return value(int): return value from main()
    '''
    (device) = dp_compat.add_parse_args()
    if isinstance(device, str) and 'sb' in device.lower():
        # User can not call SB hotplug by hand
        dp.LogErr('206 : This operation is prohibited')
        return 1

    dp.AvoidOom()
    if dp.CheckEnvironment():
        return 1

    retval = main(device)
    dp.CleanupPIDFile()
    return retval


if __name__ == '__main__':
    sys.exit(pre_main())
