#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# dp rm (remove) function
#
# Copyright(C) 2012 MUNEDA Takahiro
# Copyright(C) 2012 FUJITSU LIMITED
#

import os
import re
import sys

import dp
import ipmi
import dp_compat


def main(device):
    '''
    Call Remove() for corresponding device
    @device(str): device to be added
    Return value(int): success -> 0, error -> 1
    '''
    if not device or not isinstance(device, str):
        if isinstance(device, tuple):
            device = (device,)
        dp.LogDebug("%s: wrong device was passed" % device)
        dp.LogErr("215 : %s: invalid device is set" % device)
        return 1

    dev = device.lower()
    if re.match(r'sb\d+$', dev):
        dev_type = 'sb'
        dev_num = int(re.sub(r'^sb', '', dev))
        if not 0 <= dev_num < dp.DP_SB_MAX:
            dp.LogErr("216 : %s: invalid device is set" % device)
            return 1
    elif re.match(r'iou\d+$', dev):
        dev_type = 'iou'
        dev_num = int(re.sub(r'^iou', '', dev))
        if not 0 <= dev_num < dp.DP_IOU_MAX:
            dp.LogErr("217 : %s: invalid device is set" % device)
            return 1
    elif re.match(r'pcie\d+$', dev):
        dev_type = 'pcie'
        dev_num = int(re.sub(r'^pcie', '', dev))
        if not os.path.isdir("%s/bus/pci/slots/%s" % (dp.DP_SYSFS, dev_num)):
            dp.LogErr("218 : %s: invalid device is set" % device)
            return 1
    else:
        dp.LogErr("219 : %s: invalid device is set" % device)
        return 1

    if dev_type == 'sb':
        sb = dp.SB(dev_num)
        retval = sb.Remove()
	# Send IPMI in case of failure before ejecting SB
	if retval in [0x1, 0x2, 0x4] and ipmi.SendSBRemoveCompletion(retval):
            dp.LogErr('602 : Failed to send IPMI Command: Completion')
        if retval:
            return 1
        else:
            return 0
    elif dev_type == 'iou':
        iou = dp.IOU(dev_num)
        retval = iou.Remove()
        pci_devs = dp.ScanPciDevsOnSystem()
        if ipmi.SendBusTreeStart('IOU_REMOVE') == 0:
            ipmi.SendBusTrees(pci_devs)
            ipmi.SendBusTreeEnd('IOU_REMOVE')
        ipmi.SendMacAddresses(pci_devs)
        if ipmi.SendIOURemoveCompletion(retval):
            dp.LogErr('603 : Failed to send IPMI Command: Completion')
        if retval:
            return 1
        else:
            return 0
    elif dev_type == 'pcie':
        pcie = dp.PciHotPlugSlot(dev_num)
        return pcie.Remove()


def pre_main():
    '''
    Check command argument and system environment before processing main().
    Return value(int): return value from main()
    '''
    device = dp_compat.rm_parse_args()
    if isinstance(device, str) and 'sb' in device.lower():
        # User can not call SB hotplug by hand
        dp.LogErr('207 : This operation is prohibited')
        return 1

    dp.AvoidOom()
    if dp.CheckEnvironment():
        return 1

    retval = main(device)
    dp.CleanupPIDFile()
    return retval


if __name__ == '__main__':
    sys.exit(pre_main())
