#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# dp show function
#
# Copyright(C) 2012 MUNEDA Takahiro
# Copyright(C) 2012 FUJITSU LIMITED
#

import os
import re
import sys

import dp
import dp_compat


def main(device, possible=0):
    '''
    Call Show() for corresponding device
    @device(str): device name to be show resources
    @possible(int): if set, show a possible resource instead of current
    Return value(int): success -> 0, error -> 1
    '''
    if not device or not isinstance(device, str):
        if isinstance(device, tuple):
            device = (device,)
        dp.LogDebug("%s: wrong device was passed" % device)
        dp.LogErr("220 : %s: invalid device is set" % device)
        return 1
    if not possible:
        possible = 0
    else:
        possible = 1

    dev = device.lower()
    if re.match(r'sb\d+$', dev):
        dev_type = 'sb'
        dev_num = int(re.sub(r'^sb', '', dev))
        if not 0 <= dev_num < dp.DP_SB_MAX:
            dp.LogErr("221 : %s: invalid device is set" % device)
            return 1
    elif re.match(r'iou\d+$', dev):
        dev_type = 'iou'
        dev_num = int(re.sub(r'^iou', '', dev))
        if not 0 <= dev_num < dp.DP_IOU_MAX:
            dp.LogErr("222 : %s: invalid device is set" % device)
            return 1
    elif re.match(r'pcie\d+$', dev):
        dev_type = 'pcie'
        dev_num = int(re.sub(r'^pcie', '', dev))
        if not os.path.isdir("%s/bus/pci/slots/%s" % (dp.DP_SYSFS, dev_num)):
            dp.LogErr("223 : %s: invalid device is set" % device)
            return 1
    elif re.match(r'pcie$', dev):
        dev_type = 'pcie'
        dev_num = -1
    else:
        dp.LogErr("224 : %s: invalid device is set" % device)
        return 1

    if dev_type == 'sb':
        sb = dp.SB(dev_num)
        retval = sb.Show(possible)
    elif dev_type == 'iou':
        iou = dp.IOU(dev_num)
        retval = iou.Show()
    elif dev_type == 'pcie' and not dev_num == -1:
        pcie = dp.PciHotPlugSlot(dev_num)
        retval = pcie.Show()
    elif dev_type == 'pcie' and dev_num == -1:
        dev_nums = []
        slot_path = "%s/bus/pci/slots" % dp.DP_SYSFS

        for entry in os.listdir(slot_path):
            dev_num = int(entry)
            if dev_num & 0x1000:  # IOU's SlotNumber is 0x10[0-3][1-4]
                continue
            else:
                dev_nums.append(entry)
        if dev_nums:
            dev_nums.sort()

        retval = 0
        for dev_num in dev_nums:
            pcie = dp.PciHotPlugSlot(dev_num)
            retval += pcie.Show(show_slot_num=1)

    if retval:
        return 1
    else:
        return 0


def pre_main():
    '''
    Check command argument and system environment before processing main().
    Return value(int): return value from main()
    '''
    (device, possible) = dp_compat.show_parse_args()

    dp.AvoidOom()
    if dp.CheckEnvironment():
        return 1

    if possible:
        retval = main(device, possible=1)
    else:
        retval = main(device)
    dp.CleanupPIDFile()
    return retval


if __name__ == '__main__':
    sys.exit(pre_main())
