#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# dp stat function
#
# Copyright(C) 2012 MUNEDA Takahiro
# Copyright(C) 2012 FUJITSU LIMITED
#

import sys

import dp
import dp_compat


def main(device):
    '''
    Call PrintSomething() for corresponding device
    @device(str): device type to be shown a status
    Return value(int): success -> 0, error -> 1
    '''
    if not device or not isinstance(device, str):
        if isinstance(device, tuple):
            device = (device,)
        dp.LogDebug("%s: wrong device was passed" % device)
        dp.LogErr("225 : %s: invalid device is set" % device)
        return 1

    dev = device.lower()
    if dev == 'sb':
        return dp.PrintSBsStatus()
    elif dev == 'cpu':
        return dp.PrintCPUsStatus()
    elif dev == 'mem':
        return dp.PrintMEMsStatus()
    elif dev == 'iou':
        return dp.PrintIOUsStatus()
    elif dev == 'pcie':
        return dp.PrintPCIesStatus()
    else:
        dp.LogErr("226 : %s: invalid device is set" % device)
        return 1


def pre_main():
    '''
    Check command argument and system environment before processing main().
    Return value(int): return value from main()
    '''
    (device) = dp_compat.stat_parse_args()

    dp.AvoidOom()
    if dp.CheckEnvironment():
        return 1

    retval = main(device)
    dp.CleanupPIDFile()
    return retval


if __name__ == '__main__':
    sys.exit(pre_main())
